#include "xparameters.h"
#include "xbasic_types.h"
#include "xio.h"
#include "math.h"
#include "xintc_l.h"
#include "xuartlite_l.h"
#define  BTM1 0xFFF20000
#define  CHECK 0xFEFE0004
#define  CODEC 0xFEFE0000
#define  BTM2 0xFFF00000
#define  LEN1 131072/4
#define  LEN2 131072/4


unsigned volatile char buff;

/*
* Interrupt service routine for the UART
*/

void uart_handler(void *callback)
{
  Xuint32 IsrStatus;
  Xuint8 incoming_character;
  /* Check the ISR status register so we can identify the interrupt source */
  IsrStatus = XIo_In32(XPAR_MYUART_BASEADDR + XUL_STATUS_REG_OFFSET);
  if ((IsrStatus & (XUL_SR_RX_FIFO_FULL | XUL_SR_RX_FIFO_VALID_DATA)) != 0)
  {
    incoming_character =
      (Xuint8) XIo_In32( XPAR_MYUART_BASEADDR + XUL_RX_FIFO_OFFSET );
      buff= incoming_character;
  }
}

char rd_char()
{

  unsigned char t;
  do{
     t = buff;
   }
  while( t == 0xff);
  buff = 0xff;
  return t;
}


void record(Xuint32 addr, int len)
{
  int i,j;
  Xuint32 x;

  for(i=0;i<len;i++){
    for(j=0;j<16;j++)
      {
      while(XIo_In32(CHECK) & 0x1);
      x=XIo_In32(CODEC);
    }
    XIo_Out16(addr + (i<<2), x);
  }
}

void playback(Xuint32 addr, int len)
{
  int i,j;
  Xuint32 x;

  for(i=0;i<len;i++){
      x = XIo_In16(addr +(i<<2));
      for(j=0;j<16;j++){
	while(XIo_In32(CHECK) & 0x8);
	XIo_Out16(CODEC, x);
      }
  }
}


int main() {

  int i,j ;
  char  key;
  Xuint32 x,y;
  /* Enable UART interrupts and register uart_handler as the ISR */

  XIntc_RegisterHandler( XPAR_INTC_BASEADDR, XPAR_MYUART_DEVICE_ID,
                      (XInterruptHandler)uart_handler, (void *)0);
  XIntc_mEnableIntr( XPAR_INTC_BASEADDR, XPAR_MYUART_INTERRUPT_MASK);
  XIntc_mMasterEnable( XPAR_INTC_BASEADDR );
  XIntc_Out32(XPAR_INTC_BASEADDR + XIN_MER_OFFSET,XIN_INT_MASTER_ENABLE_MASK);
  microblaze_enable_interrupts();
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
  
  
  print("Clearing SRAM\r\n");
  for (i=0; i< LEN1 ; i++)
  { 
    XIo_Out16( BTM1+(i<<2) , 0 );
  }
  for (i=0; i<  LEN2 ; i++)
  { 
    XIo_Out16(BTM2+(i<<2), 0 );
  }
  
  print("Clearing done\r\n");

   print("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
   print("\r\n\tWELCOME TO THE DIGITAL VOICE RECORDER\r\n\n\n");
   print("Select which track you want to record/play:\r\n\n");
   print(" 1 - Track 1\r\n");
   print(" 2 - Track 2\r\n");
   print(" Once you select a track, select record or play\r\n");
   print(" p - PLAY\r\n");
   print(" r - REC\r\n");

 
   //loop to play, record, and select tracks 1 and 2  
 while(1){
   key = rd_char();
   while(1){
     while(key != '2' || key == '1'){
       if(key == '1')   
	 print(" You selected track 1\r\n");
       if(key == 'p')   
	 print(" PLAY of track 1 ended\r\n");
       if(key == 'r')   
	 print(" REC of track 1 ended\r\n");
       key = rd_char();
       if( key == 'p')
	 {
	   print("You selected to PLAY track 1\r\n");
	   playback(BTM1, LEN1);
	 }
       if( key == 'r')
	 {
	   print("You selected to RECORD track 1\r\n");
	   record(BTM1, LEN1); 
	 }
     }
     while(key != '1' || key == '2'){
       if(key == '2')   
	 print(" You selected track 2\r\n");
       if(key == 'p')   
	 print(" PLAY of track 2 ended\r\n");
       if(key == 'r')   
	 print(" REC of track 2 ended\r\n");
       key = rd_char();
       if( key == 'p')
	 {
	   print("You selected to PLAY track 2\r\n");
	   playback(BTM2, LEN2);
	 }
       if( key == 'r')
	 { 
	   print("You selected to RECORD track 2\r\n");
	   record(BTM2, LEN2);
	 }
     }
   }
 }
}
 
  

